<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\BaseExt\Controller;

/**
 * @package     imia_base_ext
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
abstract class ActionController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    /**
     * @var array
     */
    protected $fileStorage = array();

    /**
     * @var array
     */
    protected $hookObjects;

    /**
     * @param string $key
     * @return mixed
     */
    protected function getSetting($key)
    {
        if (strpos($key, '.') === false) {
            $value = $this->settings[$key];
        } else {
            $keys = explode('.', $key);

            $value = $this->settings[array_shift($keys)];
            foreach ($keys as $subKey) {
                if ($value && is_array($value)) {
                    $value = $value[$subKey];
                } else {
                    $value = null;
                    break;
                }
            }
        }

        if (is_array($value)) {
            return $this->renderTSObj($value);
        } else {
            return $value;
        }
    }

    /**
     * @param array $config
     * @return mixed
     */
    protected function renderTSObj(array $config)
    {
        if (array_key_exists('_typoScriptNodeValue', $config)) {
            $contentType = $config['_typoScriptNodeValue'];
            unset($config['_typoScriptNodeValue']);
            $contentConfig = \TYPO3\CMS\Extbase\Service\TypoScriptService::convertPlainArrayToTypoScriptArray($config);

            return $this->getContentObject()->cObjGetSingle($contentType, $contentConfig);
        } else {
            return $config;
        }
    }

    /**
     * @return \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer
     */
    protected function getContentObject()
    {
        return $this->configurationManager->getContentObject();
    }


    /**
     * Translate by key
     *
     * @param string $key
     * @param array $arguments
     * @return string
     */
    protected function translate($key, $arguments = array())
    {
        return (string)\TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate($key, $this->extensionName, $arguments);
    }

    /**
     * Load arguments for a different plugin than the current
     *
     * @param string $pluginName
     */
    protected function loadArguments($pluginName = null)
    {
        if ($pluginName) {
            $arguments = \TYPO3\CMS\Core\Utility\GeneralUtility::_GPmerged($this->getPluginNamespace($pluginName));
        } else {
            $arguments = \TYPO3\CMS\Core\Utility\GeneralUtility::array_merge_recursive_overrule($_GET, $_POST);
        }

        $this->request->setArguments($arguments);
    }

    /**
     *
     * @param string $pluginName
     * @param string $extensionName
     * @return string
     */
    protected function getPluginNamespace($pluginName = null, $extensionName = null)
    {
        $extensionService = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Service\\ExtensionService');
        $pluginNamespace = $extensionService->getPluginNamespace(
                $extensionName ? $extensionName : $this->extensionName,
                $pluginName ? $pluginName : $this->request->getPluginName()
        );

        return $pluginNamespace;
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $body
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    protected function sendMail(array $recipient, array $sender, $subject, $body, $format = 'plain', $closure = null)
    {
        $message = $this->objectManager->get('TYPO3\CMS\Core\Mail\MailMessage');

        switch ($format) {
            case 'html':
                break;
            case 'plain':
            default:
                $format = 'plain';
                break;
        }

        $message->setTo($recipient)
            ->setFrom($sender)
            ->setSubject($subject)
            ->setBody($body, 'text/' . $format);

        if ($closure) {
            try {
                $closure($message);
            } catch(\Exception $e) {}
        }

        $message->send();

        return $message->isSent();
    }

    /**
     * @param array $recipient
     * @param array $sender
     * @param string $subject
     * @param string $templateName
     * @param array $arguments
     * @param string $format
     * @param function $closure
     * @return boolean
     */
    protected function sendTemplateMail(array $recipient, array $sender, $subject, $templateName, array $arguments = array(),
        $format = 'plain', $closure = null)
    {
        switch ($format) {
            case 'html':
                $ext = 'html';
                break;
            case 'text':
            case 'plain':
            default:
                $ext = 'txt';
                break;
        }

        return $this->sendMail(
            $recipient,
            $sender,
            $subject,
            $this->renderView('Email', $templateName, $arguments, $ext),
            $format,
            $closure
        );
    }

    /**
     * @param string $name
     * @return array
     */
    protected function getFileArgument($name)
    {
        $fileArgument = null;

        if ($this->request->hasArgument($name) && array_key_exists($this->getPluginNamespace(), $_FILES)
          && $_FILES[$this->getPluginNamespace()]['name'][$name]) {
            $fileArgument = array(
                'name'      => $_FILES[$this->getPluginNamespace()]['name'][$name],
                'type'      => $_FILES[$this->getPluginNamespace()]['type'][$name],
                'tmp_name'  => $_FILES[$this->getPluginNamespace()]['tmp_name'][$name],
                'error'     => $_FILES[$this->getPluginNamespace()]['error'][$name],
                'size'      => $_FILES[$this->getPluginNamespace()]['size'][$name],
            );
        }

        return $fileArgument;
    }

    /**
     * @param array $fileData
     * @param integer $storageUid
     * @param string $folder
     * @return \TYPO3\CMS\Core\Resource\File
     */
    protected function uploadFile(array $fileData, $storageUid = 0, $folder = '')
    {
        $file = null;
        $storage = $this->getFileStorage($storageUid);

        if ($storage->isWritable() && file_exists($fileData['tmp_name'])) {
            $targetFolder = $storage->getRootLevelFolder();
            foreach (explode('/', $folder) as $folderName) {
                if ($folderName) {
                    if ($storage->hasFolderInFolder($folderName, $targetFolder)) {
                        $targetFolder = $targetFolder->getSubfolder($folderName);
                    } else {
                        $targetFolder = $targetFolder->createFolder($folderName);
                    }
                }
            }

            $file = $targetFolder->addUploadedFile($fileData, 'changeName');
            if ($file) {
                // indexes file
                $file->updateProperties(array());
            }
        }

        return $file;
    }

    /**
     * @param integer $storageUid
     * @return \TYPO3\CMS\Core\Resource\ResourceStorage
     */
    protected function getFileStorage($storageUid = 0)
    {
        if (!array_key_exists($storageUid, $this->fileStorage)) {
            $factory = $this->objectManager->get('TYPO3\CMS\Core\Resource\ResourceFactory');
            $this->fileStorage[$storageUid] = $factory->getStorageObject($storageUid);
        }

        return $this->fileStorage[$storageUid];
    }

    /**
     * @param string $controller
     * @param string $action
     * @param array $arguments
     * @param string $format
     * @return string
     */
    protected function renderView($controller, $action, $arguments, $ext = 'html')
    {
        $extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

        $templateRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName(
            $extbaseFrameworkConfiguration['view']['templateRootPath']);

        if (!$templateRootPath) {
            $templateRootPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(
                \TYPO3\CMS\Core\Utility\GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                'Resources/Private/Templates/'
            );
        }

        $templatePathAndFilename = $templateRootPath . $controller . '/' . $action . '.' . $ext;

        return $this->renderTemplate($templatePathAndFilename, $arguments);
    }

    /**
     * @param string $path
     * @param array $arguments
     * @param string $format
     * @return string
     */
    protected function renderTemplate($templatePath, $arguments, $format = 'html')
    {
        $view = $this->objectManager->create('TYPO3\CMS\Fluid\View\StandaloneView');
        $view->setFormat($format);

        $extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);

        $partialRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName(
            $extbaseFrameworkConfiguration['view']['partialRootPath']);

        if (!$partialRootPath) {
            $partialRootPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(
                \TYPO3\CMS\Core\Utility\GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                'Resources/Private/Partials/'
            );
        }
        $view->setPartialRootPath($partialRootPath);

        $layoutRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName(
            $extbaseFrameworkConfiguration['view']['layoutRootPath']);

        if (!$layoutRootPath) {
            $layoutRootPath = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath(
                \TYPO3\CMS\Core\Utility\GeneralUtility::camelCaseToLowerCaseUnderscored($this->extensionName),
                'Resources/Private/Layouts/'
            );
        }
        $view->setLayoutRootPath($layoutRootPath);

        $view->setTemplatePathAndFilename($templatePath);
        $view->assignMultiple(array_merge($arguments, array('settings' => $this->settings)));

        return $view->render();
    }

    /**
     * @return array
     */
    protected function getHookObjects()
    {
        if (!$this->hookObjects) {
            $this->hookObjects = array();
            $extKey = 'tx_' . strtolower($this->extensionName);
            $controllerName = $this->request->getControllerName() ?: ($this->controllerName ?: 'Backend');

            if (is_array($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$extKey][$controllerName])) {
                foreach ($GLOBALS['TYPO3_CONF_VARS']['EXTCONF'][$extKey][$controllerName] as $_classRef) {
                    $this->hookObjects[] = &\TYPO3\CMS\Core\Utility\GeneralUtility::getUserObj($_classRef);
                }
            }
        }

        return $this->hookObjects;
    }

    /**
     * Function: sanitize
     * Returns a sanitized string, typically for URLs.
     *
     * Parameters:
     *     $string - The string to sanitize.
     *     $forceLowercase - Force the string to lowercase?
     *     $anal - If set to *true*, will remove all non-alphanumeric characters.
     */
    protected function sanitize($string, $forceLowercase = true, $anal = false)
    {
        $strip = array("~", "`", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "_", "=", "+", "[", "{", "]",
                       "}", "\\", "|", ";", ":", "\"", "'", "&#8216;", "&#8217;", "&#8220;", "&#8221;", "&#8211;", "&#8212;",
                       "â€”", "â€“", ",", "<", ".", ">", "/", "?");

        $clean = trim(str_replace($strip, "", strip_tags($string)));
        $clean = preg_replace('/\s+/', "-", $clean);
        $clean = ($anal) ? preg_replace("/[^a-zA-Z0-9]/", "", $clean) : $clean ;

        return ($forceLowercase) ?
            (function_exists('mb_strtolower')) ?
                mb_strtolower($clean, 'UTF-8') :
                strtolower($clean) :
            $clean;
    }
}